import yaml
import os
import re
import sys
import logging
from shutil import copyfile
from pathlib import Path


def get_working_directory():
    userdir = os.path.expanduser("~")
    workingDirectory = os.path.join(userdir, "PoliCal")
    tasks_db = Path(os.path.join(workingDirectory, "tasks.db"))
    # Check for existence
    if not os.path.exists(workingDirectory):
        os.makedirs(workingDirectory)
    # Check for existence
    if not tasks_db.is_file():
        dir = os.path.dirname(__file__)
        tasks_src = os.path.join(dir, "tasks.db")
        tasks_dst = os.path.join(workingDirectory, "tasks.db")
        copyfile(tasks_src, tasks_dst)
    return workingDirectory


def get_file_location(filename):
    workingDirectory = get_working_directory()
    return os.path.join(workingDirectory, filename)


logging.basicConfig(filename=get_file_location('Running.log'), level=logging.INFO,
                    format='%(asctime)s:%(levelname)s:%(message)s')


def load_config_file(config_file_path):
    try:
        with open(get_file_location(config_file_path), 'r') as config_yaml:
            file_config = yaml.safe_load(config_yaml)
            return file_config
    except IOError:
        logging.error(
            "Archivo de configuración no encontrado, generando llaves")
        return None


def check_for_url(url):
    checker = re.search("^https.*recentupcoming$", url)
    if (checker):
        return True
    else:
        return False



from polical import MateriaClass
import csv
from polical import connectSQLite
from polical import configuration
import logging
logging.basicConfig(filename=configuration.get_file_location('Running.log'),level=logging.INFO, format='%(asctime)s:%(levelname)s:%(message)s')

try:
    with open(configuration.get_file_location('materias.csv')) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=';')
        line_count = 0
        for row in csv_reader:
            if line_count == 0:
                logging.info(f'Column names are {", ".join(row)}')
                line_count += 1
            else:
                subject = MateriaClass.Materia(row[1], row[0])
                logging.info(subject.print())
                sql = connectSQLite.saveSubjects(subject)
                for row in sql.fetchall():
                    logging.info(row)
                sql = connectSQLite.getdb().close()
except:
    logging.info("FALSE, exception ocurred")
#            line_count += 1
#    print(f'Processed {line_count} lines.')



import unittest
from polical import configuration
from selenium import webdriver

from polical from create_subject import Add_Subject_To_Trello_List
from connection import TrelloConnection


#test acceso Epn (calendario)
class TestAV(unittest.TestCase):
    def testEpn(self):
        #driver = webdriver.Chrome('C:\\drivers\\chromedriver_win32\\chromedriver.exe')
        driver = webdriver.Chrome(executable_path="C:\\drivers\\chromedriver_win32\\chromedriver.exe")
        driver.get('https://educacionvirtual.epn.edu.ec')

        titleOfEwbPage=driver.title

        self.assertTrue(titleOfEwbPage == "Google")  #true



#test acceso a trello
class TestTrel(unittest.TestCase):
    def testTrello(self):
        driver = webdriver.Chrome('C:\\drivers\\chromedriver_win32\\chromedriver.exe')
        #driver = webdriver.Chrome(executable_path="C:\drivers\chromedriver_win32\chromedriver.exe")
        driver.get('https://trello.com/es')

        titleOfEwbPage=driver.title

        self.assertTrue(titleOfEwbPage == "Trello")  #true



#test url trello
class test_check_url(unittest.TestCase):
    def test_trello(self):
        self.assertEqual(configuration.check_for_url('https://recentupcoming'), True)



#test a�adir una materia a Trello
class Test_Add_Subject_To_Trello_List(unittest.TestCase):
    def test_given_a_new_list_when_add_an_subject_then_it_has_one_element(self):
        subjectsBoard.list_lists()
	subjectsBoard.add_list(subject_name)
        self.assertEqual(1.subjectsBoard.size())



#test mostrar usuario "andres" conectado a Trello
class TestTrelloConnection(unittest.TestCase):
    def test_trello_client(self):
	trelloClient=TrelloConnection()
	user=trelloClient.initialize_trello("andres")
	self.assertEqual("andres", trello_client)



if __name__ == "__main__":
    unittest.main()



from polical import TareaClass
import csv
from polical import connectSQLite
from polical import create_subject
from polical import configuration
from datetime import datetime
import logging
logging.basicConfig(filename=configuration.get_file_location('Running.log'),level=logging.INFO, format='%(asctime)s:%(levelname)s:%(message)s')


def LoadCSVTasktoDB(username, user_dict):
    with open(configuration.get_file_location('calendar.csv')) as csv_file:
        logging.info("CSV abierto.")
        csv_reader = csv.reader(csv_file, delimiter=';')
        line_count = 0
        for row in csv_reader:
            if line_count == 0:
                line_count += 1
            elif len(row) > 9 and not line_count == 0:
                # print(len(row))
                create_subject.create(
                    Get_Subject_Name_From_CSV(row[9]),row[2], user_dict)
                sbjID = connectSQLite.getSubjectID(
                    Get_Subject_Name_From_CSV(row[9]))
                # print(row[0])
                # Siempre se extraera la fecha aun cuando pueda tener un
                # formato YMDTXXX
                task = TareaClass.Tarea(
                    row[1], row[2], row[3], datetime.strptime(row[7][0:8], '%Y%m%d'), sbjID)
                sql = connectSQLite.saveTask(task, username)
                # print("Las tareas nuevas se agregaron a la BD")
                logging.info("Las tareas nuevas se agregaron a la BD")
                sql.connection.close()


def Get_Subject_Name_From_CSV(full_subject_name):
    list = full_subject_name.split("_", 3)
    return list[1]



class Tarea:
    def __init__(self, id, title, description, due_date, subjectID):
        self.id = id
        self.title = title
        self.description = description
        self.due_date = due_date
        self.subjectID = subjectID

    def print(self):
        print(self.id, self.title, self.due_date, self.subjectID)



from datetime import datetime
from apscheduler.schedulers.blocking import BlockingScheduler

#import os
from polical import SimpleIcsToCSV
from polical import TareasCSVToBD
from polical import SendTaskToTrello

def ejecutaScript():	#Define la funcion
    SimpleIcsToCSV.convertICStoCSV()
    TareasCSVToBD.LoadCSVTasktoDB()
    SendTaskToTrello.SendTaskToTrello()

if __name__ == '__main__':
    scheduler = BlockingScheduler()
    scheduler.add_executor('processpool')
    scheduler.add_job(ejecutaScript, 'interval', seconds=3600)
    #print('Press Ctrl+{0} to exit'.format('Break' if os.name == 'nt' else 'C'))

    try:
        scheduler.start()
    except (KeyboardInterrupt, SystemExit):
        pass



import sqlite3
from polical import TareaClass
from polical import configuration

import logging
logging.basicConfig(filename=configuration.get_file_location('Running.log'), level=logging.INFO,
                    format='%(asctime)s:%(levelname)s:%(message)s')


def getdb():
    db = sqlite3.connect(configuration.get_file_location('tasks.db'))
    return db


def getCur():
    cur = getdb().cursor()
    return cur


def exec(command):
    # Use all the SQL you like
    cur = getdb().cursor()
    cur.execute(command)
    getdb().commit()
    # db.close()
    return cur


def saveTask(task, username):
    cur = getdb().cursor()
    checker = "select count(TarUID) from Tareas where TarUID = \'" + \
        task.id + "\'"
    cur.execute(checker, ())
    exists = 0
    for row in cur.fetchall():
        exists = row[0]
        # print(exists)
    if exists == 0:
        logging.info("Ejecutando..." + str(exists))
        cur.execute("INSERT INTO Tareas(TarUID, TarTitulo, TarDescripcion, TarFechaLim, Materias_idMaterias) VALUES (?, ?, ?, ?, ?);",
                    (task.id, task.title, task.description.replace('\\n', '\n'), task.due_date, task.subjectID))
        cur.connection.commit()
        idTareas = getTaskID(task.id)
        idUsuarios = getUserID(username)
        cur.execute("INSERT INTO TareasUsuarios(TarUsrEstado, idTareas, idUsuarios) VALUES (?,?,?);",
                    ("N", idTareas, idUsuarios))
    cur.connection.commit()
    return cur


def saveSubjects(subject):
    query = "INSERT INTO Materias (MatNombre, MatCodigo, MatID) values (?, ?, ?);"
    cur = getdb().cursor()
    cur.execute(query, (subject.name, subject.codigo, subject.id))
    cur.connection.commit()
    # db.close()
    return cur


def saveUser(username):
    print("Agregando usuario, "+username+"a la base de datos")
    query = "INSERT INTO Usuarios (UsrNombre) values (?);"
    cur = getdb().cursor()
    cur.execute(query, (username,))
    cur.connection.commit()
    return cur


def saveSubjectID(subject):
    cur = getdb().cursor()
    cur.execute("UPDATE Materias SET MatID = ? WHERE MatCodigo = ?;",
                (subject.id, subject.codigo))
    cur.connection.commit()
    # db.close()
    return cur


def getCardsdb(db):
    cur = exec(
        "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'Tareas';", getCur(db))
    # print all the first cell of all the rows
    cards = []
    for row in cur.fetchall():
        cards.append(row[0])
    db.close()
    return cards


def getSubjectID(subjCod):
    query = "select idMaterias from Materias where MatCodigo = \'" + subjCod + "\'"
    cur = getdb().cursor()
    cur.execute(query)
    # print all the first cell of all the rows
    sbjID = ''

    for row in cur.fetchall():
        sbjID = row[0]
    cur.connection.close()
    return sbjID


def getTaskID(TarUID):
    query = "select idTareas from Tareas where TarUID = \'" + TarUID + "\'"
    cur = getdb().cursor()
    cur.execute(query)

    idTareas = ''
    for row in cur.fetchall():
        idTareas = row[0]
    cur.connection.close()
    return str(idTareas)


def getUserID(username):
    query = "select idUsuarios from Usuarios where UsrNombre = \'" + username + "\'"
    cur = getdb().cursor()
    cur.execute(query)

    idUsuarios = ''
    for row in cur.fetchall():
        idUsuarios = row[0]
    cur.connection.close()
    return str(idUsuarios)


def getSubjectName(subjCod):
    query = "select MatNombre from Materias where MatCodigo = \'" + subjCod + "\'"
    cur = getdb().cursor()
    cur.execute(query)
    # print all the first cell of all the rows
    sbjName = ''

    for row in cur.fetchall():
        sbjName = row[0]
    cur.connection.close()
    return sbjName


def addTarTID(TarUID, TarTID, username):
    cur = getdb().cursor()
    idTareas = getTaskID(TarUID)
    idUsuarios = getUserID(username)
    cur.execute(
        "UPDATE TareasUsuarios SET TarUsrTID = ?, TarUsrEstado = ? WHERE idUsuarios = ? AND idTareas = ?;", (TarTID, "E", idUsuarios, idTareas))
    cur.connection.commit()
    # db.close()
    return cur


def getTasks(username):
    idUsuarios = getUserID(username)
    query = "select TarUsrEstado, TarUID, TarTitulo, TarDescripcion, TarFechaLim, MatID from Materias, Tareas, TareasUsuarios where Tareas.Materias_idMaterias = Materias.idMaterias AND TareasUsuarios.TarUsrEstado = 'N' AND TareasUsuarios.idTareas = Tareas.idTareas AND TareasUsuarios.idUsuarios = \'" + idUsuarios + "\';"
    cur = getdb().cursor()
    cur.execute(query)
    tasks = []
    for row in cur.fetchall():
        tasks.append(TareaClass.Tarea(row[1], row[2], row[3], row[4], row[5]))
    cur.connection.close()
    return tasks


def check_no_subjectID(subjCod):
    query = "select count(MatCodigo) from Materias where MatCodigo=\'" + \
        subjCod + "\'AND MatID=\"\";"
    cur = getdb().cursor()
    cur.execute(query)
    for row in cur.fetchall():
        result = row[0]
    cur.connection.close()
    return result


def check_user_existence(username):
    query = "select count(UsrNombre) from Usuarios where UsrNombre=\'" + \
        username + "\';"
    cur = getdb().cursor()
    cur.execute(query)
    for row in cur.fetchall():
        result = row[0]
    cur.connection.close()
    return result



from polical import TareasCSVToBD
from polical import SendTaskToTrello
from polical import SimpleIcsToCSV
from polical import configuration
import sys
from polical import Get_Trello_MoodleEPN_Keys


def main(argv):
    if len(argv) == 2:
        argument = argv[1]
        if argument == "--addUser":
            Get_Trello_MoodleEPN_Keys.onboard(False)
    else:
        users = None
        while(users == None):
            users = configuration.load_config_file("polical.yaml")
            if users == None:
                Get_Trello_MoodleEPN_Keys.onboard(False)
        for user in users.keys():
            SimpleIcsToCSV.convertICStoCSV(users[user]['calendar_url'])
            TareasCSVToBD.LoadCSVTasktoDB(user, users[user])
            SendTaskToTrello.SendTaskToTrello(user, users[user])


if __name__ == "__main__":
    main(sys.argv)



import wget
import os
from polical import configuration
import logging
logging.basicConfig(filename=configuration.get_file_location('Running.log'),level=logging.INFO, format='%(asctime)s:%(levelname)s:%(message)s')


def convertICStoCSV():
    logging.info("Empezando:")
    logging.infoc("Eliminando si existe")
    print("Empezando:")
    print("Eliminando si existe")
    filename="mycalendar.ics"
    if os.path.exists(filename):
        os.remove(filename)
    url = "https://educacionvirtual.epn.edu.ec/calendar/export_execute.php?userid=7587&authtoken=a43c2f67460752ab1e1b0d5a784dd330cb5b93e7&preset_what=all&preset_time=recentupcoming"
    wget.download(url, "mycalendar.ics")
    f = open("mycalendar.ics","r")
    f2 = open("calendar.csv","w+")
    f1 = f.readlines()
    headers = ["BEGIN","UID","SUMMARY","DESCRIPTION","CLASS","LAST-MODIFIED","DTSTAMP","DTSTART","DTEND","CATEGORIES"]
    for x in headers:
        f2.write(x+";")
    f2.write("\n")
    wrBegin=False
    wrNormal=False
    wrDescription=False
    for x in f1:
        #print(x)
        #print(str(x.count('\t'))+"---"+x)
        list = x.split(":",1)
        if list[0]=="BEGIN" and list[1]=="VEVENT\n":
            wrNormal=True
            wrBegin=True
            list[1]="VEVENT"
            #print(list[1])
        elif list[0]=="DESCRIPTION":
            wrDescription=True
            f2.write("\"")
            #print(list[0])
        elif list[0]=="CLASS":
            wrDescription=False
            f2.write("\""+";")
            #print(list[0])
        elif list[0]=="END" and list[1]=="VEVENT\n":
            wrNormal=False
            #print(list[0])
        else:
            ""
        if wrNormal and wrDescription==False:
            removebsn=list[1].split("\n",1)
            f2.write(removebsn[0]+";")
        elif wrNormal and wrDescription:
            for y in list:
                new_list = {x.replace('\n', '').replace('\t', '') for x in list}
                for x in new_list:
                    f2.write(x)
        elif wrNormal==False and wrBegin:
            f2.write("\n")



from polical import TareasCSVToBD
from polical import SendTaskToTrello
import SimpleIcsToCSV
from polical import configuration
users = configuration.load_config_file("polical.yaml")
for user in users.keys():
    SimpleIcsToCSV.convertICStoCSV(users[user]['calendar_url'])
    TareasCSVToBD.LoadCSVTasktoDB(user, users[user])
    SendTaskToTrello.SendTaskToTrello(user, users[user])



from trello import TrelloClient
from polical import connectSQLite
from polical import configuration
from datetime import datetime
import logging
logging.basicConfig(filename=configuration.get_file_location('Running.log'),level=logging.INFO, format = '%(asctime)s:%(levelname)s:%(message)s')


def SendTaskToTrello(username, user_dict):
    #user_dict = configuration.load_config_file('polical.yaml')
    client = TrelloClient(
        api_key=user_dict['api_key'],
        api_secret=user_dict['api_secret'],
        token=user_dict['oauth_token'],
        token_secret=user_dict['oauth_token_secret'],
    )
    member_id = user_dict['owner_id']
    subjectsBoard = client.get_board(user_dict['board_id'])
    tasks = connectSQLite.getTasks(username)
    if len(tasks) == 0:
        logging.info("No existen tareas nuevas, verifique consultando el calendario")
        print("No existen tareas nuevas, verifique consultando el calendario")
    else:
        for x in tasks:
            logging.info("Agregando Tarea:" + x.subjectID +" "+ x.title + " " + x.description + " " + x.due_date)
            print("Agregando Tarea:")
            x.print()
            subjectList = subjectsBoard.get_list(x.subjectID)
            card = subjectList.add_card(
                x.title, x.description.replace('\\n', '\n'))
            card.assign(member_id)
            x.due_date = x.due_date[0:10] + " 07:00:00"
            card.set_due(datetime.strptime(x.due_date, '%Y-%m-%d %H:%M:%S'))
            # print(x.due_date)
            # card.set_due(x.due_date)
            connectSQLite.addTarTID(x.id, subjectList.list_cards()[-1].id, username)



import wget
import os
import csv
import sys
from polical import configuration

import logging
logging.basicConfig(filename=configuration.get_file_location('Running.log'),level=logging.INFO, format='%(asctime)s:%(levelname)s:%(message)s')

def addEvent(header, filename):
    f = open(configuration.get_file_location(filename), "r", encoding='utf-8')
    f2 = open(configuration.get_file_location("calendar.csv"), "w+")
    f1 = f.readlines()
    # print(header)
    for x in header:  # Write the defenitive header
        f2.write(x)
        if x != "END":
            f2.write(";")
    f2.write("\n")
    wrBegin = False  # Flag to detect the line BEGIN:VEVENT
    # , and start saving the parameters of the event
    # in this case is for getting the headers for the CSV
    wrNormal = False  # Flag to say that is writting everything but no description
    wrDescription = False  # Flag to say that is writting a description
    for x in f1:  # Read all of the lines between BEGIN:VEVENT and END:VEVENT
        # Separate the tags from the content, the results are limited to 2
        list = x.split(":", 1)
        # list for looking the first character on the line
        chars = [c for c in list[0]]
        if list[0] == "BEGIN" and list[1] == "VEVENT\n":  # Here an event begins
            wrNormal = True  # Flag activated, for ONE EVENT
            wrBegin = True  # Flag activated, for the whole set of events
            list[1] = "VEVENT"  # ERASED the "\n" character
        # Here a DESCRIPTION begins, can have a lot of lines.
        elif list[0] == "DESCRIPTION":
            wrDescription = True  # Flag activated, for the DESCRIPTION
            # A description can be conformed by a lot of especial characters so
            # the description will be between ""
            f2.write("\"")
        # If the lines are not beginning  with an space or an "\t", means that
        # is another tag and not currently a decription, but it can change
        elif chars[0] != ' ' and chars[0] != '\t' and chars[0] != '\n' and wrDescription:
            wrDescription = False  # Flag deactivated, it means a description content ends
            f2.write("\"" + ";")  # Also finished with a semicolon
        elif list[0] == "END" and list[1] == "VEVENT\n":  # An event fisnish
            wrNormal = False  # Flag deactivated, a event ends
        else:
            ""

        if wrNormal and wrDescription == False:
            # If wrNormal is activated and wrDescription deactivated means that
            # is every posible tag except a description
            # print(list)
            try:
                # Delete the especial character
                removebsn = list[1].split("\n", 1)
                f2.write(removebsn[0].replace(';','') + ";")  # Adds semicolon and avoids double ";"
            except Exception as e:
                print(e)
        elif wrNormal and wrDescription:  # If wrNormal and wrDescription are
            # activated it is a description and can have differentes kind of
            # lines
            for y in list:  # A for loop for replacing special characters as
                # "\n", "\t" and the word "DESCRIPTION"
                new_list = {x.replace('\n', '').replace('\t', '').replace('DESCRIPTION', '')
                            for x in list}
            for x in new_list:  # A for loop to add the contents to the description
                f2.write(x)
        elif wrNormal == False and wrBegin:
            # if wrNormal is deactivated and wrBegin activated, that means that
            # a event has ended so a "\n" is added
            f2.write("\n")


def convertICStoCSV(url):
    # print("Empezando:")
    # print("Eliminando si existe")
    print("Descargando calendario desde Aula Virtual...")
    logging.info("Descargando calendario desde Aula Virtual...")
    filename = configuration.get_file_location("mycalendar.ics")
    if os.path.exists(filename):
        os.remove(filename)
    # aurl = configuration.load_config_file('polical.yaml')['calendar_url']
    wget.download(url, filename)
    addEvent(findHeader(filename), filename)
    print("\nEspere...")
    logging.info("Descarga de calendario finalizada.")

def findHeader(icsCal):
    f = open(configuration.get_file_location(icsCal), "r", encoding="utf-8")
    # print("Looking for headers in this file....")
    f2 = open(configuration.get_file_location("calendar.csv"), "w+")
    f1 = f.readlines()
    wrBegin = False  # Flag to detect the line BEGIN:VEVENT
    # , and start saving the parameters of the event
    # in this case is for getting the headers for the CSV
    wrNormal = False
    wrDescription = False  # Saving a description that could be large
    for x in f1:
        # print(x)
        # print(str(x.count('\t'))+"---"+x)
        # headers = []
        list = x.split(":", 1)
        chars = [c for c in list[0]]
        # Looking for the line that begins an event
        if list[0] == "BEGIN" and list[1] == "VEVENT\n":
            wrNormal = True
            wrBegin = True
        # Looking for the line where the description begins to activate its
        # flag
        elif list[0] == "DESCRIPTION":
            wrDescription = True
        # If the lines are not beginning  with an space or an "\t", means that
        # is another tag and not currently a decription, but it can change
        elif chars[0] != ' ' and chars[0] != '\t' and chars[0] != '\n' and wrDescription:
            wrDescription = False

        if list[0] == "END":  # If the event comes to its end the flag
            # deactivates
            wrNormal = False
        else:
            ""

        # If all of the headers are reached it only writes and \n and stops the for loop
        if wrNormal == False and wrBegin == True:
            f2.write("END\n")
        elif wrNormal and wrDescription == False:
            # Everything that can be and tag inside an event is appended to the
            # list, but if the events are irregulars this can cause errors
            f2.write(list[0].replace(';', '') + ";")
        elif wrNormal and wrDescription:
            # If the tag is description this is added, but can be lines that
            # are part of the description
            if list[0] == "DESCRIPTION":
                f2.write(list[0].replace(';', '') + ";")

    f2.close()  # File calendar.csv is closed
    listHeaders = []  # A list of every posible header if the ics file is
    # IRREGULAR

    # Reading the calendar.csv temporary to get the header with the most number
    # of tags
    with open(configuration.get_file_location("calendar.csv")) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=';')
        # line_count = 0
        for row in csv_reader:
            listHeaders.append(row)

    # File name to erase the calendar.csv that is temporary
    filename = configuration.get_file_location("calendar.csv")
    if os.path.exists(filename):
        os.remove(filename)
    # Get the header with the most number of tags
    return max(listHeaders, key=len)


def main(argv):
    if len(argv) == 2:  # Arguments
        filename = argv[1]
    else:
        print("python icsReader.py file/location/file.ics")
        logging.info("python icsReader.py file/location/file.ics")
    addEvent(findHeader(filename), filename)


if __name__ == "__main__":
    main(sys.argv)



# This code is based on https://github.com/delucks/gtd.py onboard function,
# especial thanks to: delucks
import yaml
import os
import webbrowser
import trello
import sys
from polical import configuration
from polical import connectSQLite
from requests_oauthlib import OAuth1Session
from requests_oauthlib.oauth1_session import TokenRequestDenied
import logging
logging.basicConfig(filename=configuration.get_file_location('Running.log'), level=logging.INFO,
                    format='%(asctime)s:%(levelname)s:%(message)s')


class DevNullRedirect:
    '''Temporarily eat stdout/stderr to allow no output.
    This is used to suppress browser messages in webbrowser.open'''

    def __enter__(self):
        self.old_stderr = os.dup(2)
        self.old_stdout = os.dup(1)
        os.close(2)
        os.close(1)
        os.open(os.devnull, os.O_RDWR)

    def __exit__(self, exc_type, exc_value, traceback):
        os.dup2(self.old_stderr, 2)
        os.dup2(self.old_stdout, 1)


def onboard(no_open, output_path='polical.yaml'):
    '''Obtain Trello API credentials and put them into your config file.
    This is invoked automatically the first time you attempt to do an operation which requires authentication.
    The configuration file is put in an appropriate place for your operating system. If you want to change it later,
    you can use `gtd config -e` to open it in $EDITOR.
    '''
    username = ""
    output_file = configuration.get_file_location(output_path)  # Use platform detection
    user_api_key_url = 'https://trello.com/app-key'
    request_token_url = 'https://trello.com/1/OAuthGetRequestToken'
    authorize_url = 'https://trello.com/1/OAuthAuthorizeToken'
    access_token_url = 'https://trello.com/1/OAuthGetAccessToken'
    calendar_moodle_epn_url = 'https://educacionvirtual.epn.edu.ec/calendar/view.php?view=upcoming&course=1'
    # First, open the URL that allows the user to get an auth token. Tell them to copy both into the program
    logging.info("Mostrando print-board al usuario")
    print('Bienvenido a PoliCal!\n Recuerde que antes de iniciar el proceso de obtención de credenciales ud debe tener una cuenta en Trello y en el Aula Virtual, y deben estar iniciadas las sesiones en el navegador predeterminado')
    print('')
    print("\n\n")
    username = input("Ingrese su nombre:")
    print("PASO 1: Acceso a Trello")
    print("Recuerde ingresar e iniciar sesion previamente a www.trello.com")
    print("En su navegador web se cargará el siguiente URL:")
    print('  ' + user_api_key_url)
    print('Cuando llegue a esa página, inicie sesión y copie la "Tecla" o "Key" que se muestra en un cuadro de texto.')
    print('Si es la primera vez que realiza este proceso debe aceptar los terminos y condiciones de Trello')
    input("Presione ENTER para ir al enlace")
    if not no_open:
        with DevNullRedirect():
            webbrowser.open_new_tab(user_api_key_url)
    api_key = input('Por favor, introduzca el valor de "Tecla" o "Key":')
    print('Ahora desplácese hasta la parte inferior de la página y copie el "Secreto" o "Secret" que se muestra en un cuadro de texto.')
    api_secret = input('Por favor, introduzca el valor de "Secret":')
    # Then, work on getting OAuth credentials for the user so they are permanently authorized to use this program
    print("\n\n")
    print("PASO 2: Permitir acceso de Polical a Trello")
    print('Ahora obtendremos las credenciales de OAuth necesarias para usar este programa...')
    # The following code is cannibalized from trello.util.create_oauth_token from the py-trello project.
    # Rewriting because it does not support opening the auth URLs using webbrowser.open and since we're using
    # click, a lot of the input methods used in that script are simplistic compared to what's available to us.
    # Thank you to the original authors!
    '''Step 1: Get a request token. This is a temporary token that is used for
    having the user authorize an access token and to sign the request to obtain
    said access token.'''
    session = OAuth1Session(client_key=api_key, client_secret=api_secret)
    try:
        response = session.fetch_request_token(request_token_url)
    except TokenRequestDenied:
        print(
            'Invalid API key/secret provided: {0} / {1}'.format(api_key, api_secret))
        sys.exit(1)
    resource_owner_key, resource_owner_secret = response.get(
        'oauth_token'), response.get('oauth_token_secret')
    '''Step 2: Redirect to the provider. Since this is a CLI script we do not
    redirect. In a web application you would redirect the user to the URL
    below.'''
    user_confirmation_url = '{authorize_url}?oauth_token={oauth_token}&scope={scope}&expiration={expiration}&name={name}'.format(
        authorize_url=authorize_url,
        oauth_token=resource_owner_key,
        expiration='never',
        scope='read,write',
        name='PoliCal',
    )
    print('Visite la siguiente URL en su navegador web para autorizar a PoliCal acceso a su cuenta:')
    print('  ' + user_confirmation_url)
    print("Concedale los permisos a PoliCal para acceder a sus datos de Trello, estas credenciales se mantendran de manera local")
    input("Presione ENTER para ir al enlace")
    if not no_open:
        with DevNullRedirect():
            webbrowser.open_new_tab(user_confirmation_url)
    '''After the user has granted access to you, the consumer, the provider will
    redirect you to whatever URL you have told them to redirect to. You can
    usually define this in the oauth_callback argument as well.'''
    confirmation = input(
        '¿Has autorizado a PoliCal? Escriba n para no y S para si:')
    while confirmation == "n":
        confirmation = input(
            '¿Has autorizado a PoliCal? Escriba n para no y S para si:')

    oauth_verifier = input('¿Cuál es el código de verificación?:').strip()
    '''Step 3: Once the consumer has redirected the user back to the oauth_callback
    URL you can request the access token the user has approved. You use the
    request token to sign this request. After this is done you throw away the
    request token and use the access token returned. You should store this
    access token somewhere safe, like a database, for future use.'''
    session = OAuth1Session(
        client_key=api_key,
        client_secret=api_secret,
        resource_owner_key=resource_owner_key,
        resource_owner_secret=resource_owner_secret,
        verifier=oauth_verifier,
    )
    access_token = session.fetch_access_token(access_token_url)

    '''Step 4: Ahora se debe buscar el calendario en formato ICS generado por el
    Aula Virtual'''
    print("\n\n")
    print("PASO 3: Obtención del calendario del Aula Virtual")
    print("Recuerde ingresar e iniciar sesion previamente a https://educacionvirtual.epn.edu.ec/")
    print("A continuación se abrirá un link hacia el Aula Virtual EPN, en proximos eventos para: elija Todos los cursos")
    print("y a continuación desplácese a la parte más inferior de la página y de clic en el botón Exportar Calendario")
    print("Luego, en la opción Exportar seleccione todos los eventos y después en \"para\" seleccione los 60 días recientes y próximos")
    print("Finalmente de clic en el boton Obtener URL del calendario")
    print('Visite la siguiente URL en su navegador web para obtener el calendario del aula virtual EPN:')
    print('  ' + calendar_moodle_epn_url)
    input("Presione ENTER para ir al enlace e iniciar el proceso, no olvide verificar si su sesión del Aula Virtual se encuentra activa")
    if not no_open:
        with DevNullRedirect():
            webbrowser.open_new_tab(calendar_moodle_epn_url)
    calendar_url = calendar_moodle_epn_url
    while not(configuration.check_for_url(calendar_url)):
        calendar_url = input(
            'Por favor, introduzca el url generado por el Aula Virtual, si este es erróneo se volverá a solicitar:')
    final_output_data = {
        'oauth_token': access_token['oauth_token'],
        'oauth_token_secret': access_token['oauth_token_secret'],
        'api_key': api_key,
        'api_secret': api_secret,
        'calendar_url': calendar_url,
    }
    print("\n\n")
    # Ensure we have a folder to put this in, if it's in a nested config location
    """
    output_folder = os.path.dirname(output_file)
    if not os.path.isdir(output_folder):
        os.makedirs(output_folder)
        print('Created folder {0} to hold your configuration'.format(output_folder))
    # Try to be safe about not destroying existing credentials
    """
    board_id, owner_id = get_working_board_id(
        api_key, api_secret, access_token['oauth_token'], access_token['oauth_token_secret'])
    final_output_data['board_id'] = board_id
    final_output_data['owner_id'] = owner_id
    if check_file_existence(output_file):
        check_user_on_file(output_file, username + owner_id)
        with open(output_file) as file:
            # use safe_load instead load
            super_final = yaml.safe_load(file)
            super_final[username + owner_id] = final_output_data
            connectSQLite.saveUser(username + owner_id)
    else:
        super_final = {
            username + owner_id: final_output_data
        }
        connectSQLite.saveUser(username + owner_id)
    with open(output_file, 'w') as f:
        f.write(yaml.safe_dump(super_final, default_flow_style=False))
    #print('Las credenciales se guardaron en "{0}"- ahora puedes utilizar PoliCal'.format(output_file))
    #print('Use the "config" command to view or edit your configuration file')


def get_working_board_id(api_key, api_secret, oauth_token, oauth_token_secret):
    client = trello.TrelloClient(
        api_key=api_key,
        api_secret=api_secret,
        token=oauth_token,
        token_secret=oauth_token_secret,
    )
    board_id = ''
    all_boards = client.list_boards()
    for board in all_boards:
        if board.name == "TareasPoli":
            board_id = board.id
    if board_id == '':
        logging.error(
            ("No se encontró el board \"TareasPoli\", será creado ahora"))
        print("No se encontró el board \"TareasPoli\", será creado ahora")
        client.add_board("TareasPoli")
        all_boards = client.list_boards()
        for board in all_boards:
            if board.name == "TareasPoli":
                board_id = board.id
    return board_id, board.all_members()[-1].id


def check_user_on_file(output_file, username):
    if os.path.exists(output_file):
        with open(output_file) as file:
            # use safe_load instead load
            users = yaml.safe_load(file)
            if username in users.keys():
                overwrite = input(
                    'El nombre de usuario actualmente ya existe, sobreescribir? s/N:')
                if overwrite == 'N':
                    return


def check_file_existence(output_file):
    if os.path.exists(output_file):
        with open(output_file) as file:
            # use safe_load instead load
            users = yaml.safe_load(file)
            if len(users.keys()) > 0:
                return True
            else:
                return False
    else:
        return False
# onboard(True)



class Materia:
    def __init__(self, name, codigo, id=""):
        self.id = id
        self.name = name
        self.codigo = codigo

    def print(self):
        print(self.id, self.name, self.codigo)


from polical import MateriaClass
from trello import TrelloClient
from polical import connectSQLite
import logging
from polical import configuration
logging.basicConfig(filename=configuration.get_file_location('Running.log'),level=logging.INFO,
                    format='%(asctime)s:%(levelname)s:%(message)s')

def create(subjCod, task_title, user_dict):
    # config = configuration.load_config_file('polical.yaml')
    client = TrelloClient(
        api_key=user_dict['api_key'],
        api_secret=user_dict['api_secret'],
        token=user_dict['oauth_token'],
        token_secret=user_dict['oauth_token_secret'],
    )
    subjectsBoard = client.get_board(user_dict['board_id'])
    if(connectSQLite.check_no_subjectID(subjCod) == 1):
        subject_name = connectSQLite.getSubjectName(subjCod)
        logging.info(subject_name)
        print(subject_name)
        id = ""
        Add_Subject_To_Trello_List(subjectsBoard, subject_name, subjCod)
    elif(connectSQLite.getSubjectName(subjCod) == ""):
        logging.info(
            "Nombre de materia no encontrado, titulo de la tarea:"+ str(task_title))
        print("\n Nombre de materia no encontrado, titulo de la tarea:"+ task_title)
        logging.info("Por favor agregue el nombre de la materia:")
        subject_name = input("Por favor agregue el nombre de la materia:")
        logging.info(subject_name)
        response = "N"
        while response == "N" or response == "n":
            logging.info("¿El nombre de la materia " + subject_name +" es correcto?")
            print("¿El nombre de la materia " + subject_name +" es correcto?")
            logging.info("¿Guardar? S/N:")
            response = input("¿Guardar? S/N:")
            logging.info(response)
        subject = MateriaClass.Materia(subject_name, subjCod)
        connectSQLite.saveSubjects(subject)
        Add_Subject_To_Trello_List(subjectsBoard, subject_name, subjCod)


def Add_Subject_To_Trello_List(subjectsBoard, subject_name, subjCod):
    id = ""
    for x in subjectsBoard.list_lists():
        if x.name == subject_name:
            id = x.id
    if id == "":
        subjectsBoard.add_list(subject_name)
        for x in subjectsBoard.list_lists():
            if x.name == subject_name:
                id = x.id
    subject = MateriaClass.Materia(subject_name, subjCod, id)
    logging.info(subject.print())
    sql = connectSQLite.saveSubjectID(subject)
    for row in sql.fetchall():
        logging.info("fila: " + str(row))
    sql = connectSQLite.getdb().close()



import setuptools
from polical import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="polical",
    version=__version__,
    author="Andrés Andrade",
    author_email="luis.andradec14@gmail.com",
    description="Este es un programa hecho en python para sincronizar tareas existentes en el aula virtual de moodle EPN con una cuenta de Trello.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='BSD-3',
    url="https://github.com/andr3slelouch/PoliCal/",
    packages=setuptools.find_packages(),
    install_requires=[
    'certifi',
    'chardet',
    'idna',
    'oauthlib',
    'py-trello',
    'python-dateutil',
    'pytz',
    'PyYAML',
    'requests',
    'requests-oauthlib',
    'six',
    'urllib3',
    'wget'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Environment :: Console"
    ],
    include_package_data=True,
    python_requires='>=3.5',
)



import trello
import requests
from polical import configuration
import logging
logging.basicConfig(filename=configuration.get_file_location('Running.log'),level=logging.INFO, format='%(asctime)s:%(levelname)s:%(message)s')

class GTDException(Exception):
    '''single parameter indicates exit code for the interpreter, because
    this exception typically results in a return of control to the terminal'''

    def __init__(self, errno):
        self.errno = errno

class TrelloConnection:
    '''this one handles connection, retry attempts, stuff like that so it doesn't bail out each
    time we lose connection
    creating a connection requires configuration to be parsed because we need the api keys- so this will need to be invoked
    after the config parser is done doing its thing, with a parameter perhaps being the config

    :param bool autoconnect: should we make a network connection to Trello immediately?
    '''

    def __init__(self, config, autoconnect=True):
        self.autoconnect = autoconnect
        self.config = config
        self.trello = self.__connect(config) if autoconnect else None

    def __connect(self, config):
        trello_client = self.initialize_trello(config)
        try:
            # A simple API call (data reused in BoardTool.get_main_board) to initiate connection & test our credentials etc
            self.boards = trello_client.fetch_json('/members/me/boards/?filter=open')
            return trello_client
        except requests.exceptions.ConnectionError:
            logging.critical('[FATAL] Could not connect to the Trello API!')
            raise GTDException(1)
        except trello.exceptions.Unauthorized:
            logging.critical('[FATAL] Trello API credentials are invalid')
            raise GTDException(1)

    def __repr__(self):
        c = 'disconnected' if self.trello is None else 'connected'
        # isaac: se corrigio: return 'TrelloConnection {0} at {0}'.format(c, id(self))
        return 'TrelloConnection {0} at {1}'.format(c, id(self))

    def __str__(self):
        return repr(self)

    def initialize_trello(self, config):
        '''Initializes our connection to the trello API
        :param dict config: parsed configuration from the yaml file
        :returns: trello.TrelloClient client
        '''
        trello_client = trello.TrelloClient(
            api_key=config.api_key,
            api_secret=config.api_secret,
            token=config.oauth_token,
            token_secret=config.oauth_token_secret,
        )
        return trello_client
